//================================================================================
//
//  This code is the intellectual property of Jerry Jongerius (duckware.com)
//
//================================================================================

//
//  Push a potentially blocking 'print' into its own Thread (outside of the
//  tight loop that reads from a socket).
//

import java.io.*;

public class Printer implements Runnable {
    private boolean m_bRunning=true;
    private Lock m_lock = new Lock();
    private PrintStream m_out;
    private String m_s="";

    //----------------------------------------------------------------------
    public Printer( PrintStream out ) {
        m_out = out;
        (new Thread(this)).start();
        }
    //----------------------------------------------------------------------
    public void print( String s ) {
        synchronized(m_lock) {
            m_s += s;
            }
        G.notify(m_lock);
        }
    //----------------------------------------------------------------------
    public void run() {
        while (m_bRunning) {
            flush();
            G.wait( m_lock );
            }
        }
    //----------------------------------------------------------------------
    public void end() {
        m_bRunning = false;
        G.notify(m_lock);
        flush();
        }
    //----------------------------------------------------------------------
    private void flush() {
        String s;
        synchronized(m_lock) {
            s = m_s;
            m_s = "";
            }
        m_out.print( s );
        }
    }